==> builders_inventory-2.2.4-sources.jar: _3650/builders_inventory/feature/hotbar_swapper/HotbarSwapper.java <==
package _3650.builders_inventory.feature.hotbar_swapper;

import _3650.builders_inventory.BuildersInventory;
import _3650.builders_inventory.ModKeybinds;
import _3650.builders_inventory.config.Config;
import _3650.builders_inventory.feature.extended_inventory.ExtendedInventory;
import _3650.builders_inventory.mixin.feature.hotbar_swapper.GuiGraphicsAccessor;
import _3650.builders_inventory.mixin.feature.hotbar_swapper.GuiInvoker;
import net.minecraft.class_10799;
import net.minecraft.class_1661;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_746;
import net.minecraft.class_9779;

public class HotbarSwapper {
	
	private static final class_2960 SPRITE_HOTBAR = class_2960.method_60656("hud/hotbar");
	private static final class_2960 SPRITE_HOTBAR_SELECTION_WIDE = BuildersInventory.modLoc("hud/hotbar_selection_wide");
	
	public static boolean selecting = false;
	public static boolean singleColumn = false;
	public static int row = 0;
	public static int queueSwap = 0;
	
	public static int max = 4;
	private static int extendedOffset = 2;
	private static int hudShift = (max - 1) * -22 - extendedOffset;
	
	/**
	 * Refreshes the max rows
	 */
	public static void refresh() {
		ExtendedInventory.refresh();
		max = ExtendedInventory.enabled
				&& ExtendedInventory.PAGE_CONTAINER.canModify()
				&& Config.instance().hotbar_swapper_includeExtendedInventory ? 10 : 4;
		extendedOffset = Config.instance().hotbar_swapper_extendedInventoryOffset;
		hudShift = (max - 1) * -22 - extendedOffset;
	}
	
	private static void reset() {
		selecting = false;
		singleColumn = false;
		row = 0;
		queueSwap = 0;
		refresh();
	}
	
	/*
	 * RENDERING
	 */
	
	public static void renderHotbars(class_332 gui, class_310 mc) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			
			if (singleColumn) {
				int offset = mc.field_1724.method_31548().method_67532() * 20;
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 - (i * 22);
					if (i > 3) y -= extendedOffset;
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 182, 22, 0, 0, width - 91 + offset, y, 1, 22);
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 182, 22, 1 + offset, 0, width - 91 + offset + 1, y, 20, 22);
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 181, 22, 180, 0, width - 91 + offset + 21, y, 1, 22);
				}
			} else {
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 - (i * 22);
					if (i > 3) y -= extendedOffset;
					gui.method_52706(class_10799.field_56883, SPRITE_HOTBAR, width - 91, y, 182, 22);
				}
			}
		}
	}
	
	public static boolean renderHotbarSelector(class_332 gui, class_310 mc, class_2960 selectorSprite) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			if (singleColumn) {
				var sprite = ((GuiGraphicsAccessor) gui).getSprites().method_18667(selectorSprite).method_45851();
				var spriteWidth = sprite.method_45807();
				var spriteHeight = sprite.method_45815();
				
				int offset = mc.field_1724.method_31548().method_67532() * 20;
				int y = gui.method_51443() - 22 - 1 - (row * 22);
				if (row > 3) y -= extendedOffset;
				gui.method_52706(class_10799.field_56883, selectorSprite, width - 91 - 1 + offset, y, spriteWidth, spriteHeight);
			} else {
				int y = gui.method_51443() - 22 - 1 - (row * 22);
				if (row > 3) y -= extendedOffset;
				gui.method_52706(class_10799.field_56883, SPRITE_HOTBAR_SELECTION_WIDE, width - 91 - 1, y, 184, 24);
			}
			
			return true;
		}
		return false;
	}
	
	public static void renderItems(class_332 gui, class_9779 deltaTick, class_310 mc, GuiInvoker hud) {
		if (selecting) {
			int width =  gui.method_51421() / 2;
			class_746 player = mc.field_1724;
			
			if (singleColumn) {
				int x = width - 91 + 3 + (player.method_31548().method_67532() * 20);
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 + 3 - (i * 22);
					if (i > 3) y -= extendedOffset;
					hud.callRenderSlot(gui, x, y, deltaTick, player, ExtendedInventory.getItem(mc, getSlot(i, player.method_31548().method_67532())), x + (y * 22));
				}
			} else {
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 + 3 - (i * 22);
					if (i > 3) y -= extendedOffset;
					for (int j = 0; j < 9; j++) {
						int x = width - 91 + 3 + (j * 20);
						hud.callRenderSlot(gui, x, y, deltaTick, player, ExtendedInventory.getItem(mc, getSlot(i, j)), x * y);
					}
				}
			}
		}
	}
	
	public static void renderLabels(class_332 gui, class_310 mc) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			
			int x = singleColumn ? (width - 91 - 7 + (mc.field_1724.method_31548().method_67532() * 20)) : (width - 91 - 7);
			for (int i = 1; i < max; i++) {
				int y = gui.method_51443() - 22 + 7 - (i * 22);
				if (i > 3) y -= extendedOffset;
				gui.method_51433(mc.field_1772, String.valueOf(i), x, y, 0xFFFFFFFF, true);
			}
		}
	}
	
	public static class_1799 toolHighlightOverride(class_1799 stack, class_310 mc, GuiInvoker hud) {
		if (selecting) {
			class_1799 main = getMainItem(mc);
			if (!main.method_7960()) hud.setToolHighlightTimer((int)(40.0 * mc.field_1690.method_48191().method_41753()));
			return main;
		}
		return stack;
	}
	
	public static void shiftHud(class_332 gui) {
		if (selecting) {
			gui.method_51448().pushMatrix();
			gui.method_51448().translate(0, hudShift);
		}
	}
	
	public static void shiftHudReset(class_332 gui) {
		if (selecting) {
			gui.method_51448().popMatrix();
		}
	}
	
	/*
	 * LOGIC
	 */
	
	public static void preKeybinds(class_310 mc) {
		if (selecting && Config.instance().hotbar_swapper_useHotbarHotkeys) {
			// Hotbar keys
			// Consumes all hotbar keys regardless of capacity because I need to cancel them anyways
			for (int i = 0; i < 9; ++i) {
				if (mc.field_1690.field_1852[i].method_1436() && i < max) {
					row = i + 1;
					queueSwap = Config.instance().hotbar_swapper_hotkeyDelay + 1;
				}
			}
		}
	}
	
	public static void clientTick(class_310 mc) {
		if (mc.field_1724 == null) return;
		if (!mc.field_1724.method_68878()) {
			if (selecting) reset();
			return;
		}
		
		if (queueSwap > 0 && --queueSwap == 0) swapRow(mc);
		
		boolean fullKey = ModKeybinds.HOTBAR_SCROLL.method_1434();
		boolean fullKeyPressed = ModKeybinds.HOTBAR_SCROLL.method_1436();
		boolean columnKey = ModKeybinds.COLUMN_SCROLL.method_1434();
		boolean columnKeyPressed = ModKeybinds.COLUMN_SCROLL.method_1436();
		var behavior = Config.instance().hotbar_swapper_behavior;
		
		switch (behavior) {
		case PREFER_FULL:
			if (!Config.instance().hotbar_swapper_stickyModifiers) singleColumn = columnKey;
			if (fullKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = columnKey;
				return;
			}
			if (selecting && !fullKey) swapRow(mc);
			break;
		case PREFER_COLUMN:
			if (!Config.instance().hotbar_swapper_stickyModifiers) singleColumn = !fullKey;
			if (columnKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = !fullKey;
				return;
			}
			if (selecting && !columnKey) swapRow(mc);
			break;
		case SEPARATE:
			if (selecting) {
				if ((!columnKey && singleColumn) || !fullKey && !singleColumn) swapRow(mc);
				return;
			}
			
			if (fullKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = false;
				return;
			}
			if (columnKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = true;
				return;
			}
			break;
		}
	}
	
	public static boolean mouseScroll(double amount) {
		if (selecting) {
			row += amount;
			row %= max;
			if (row < 0) row = max + row; //row is negative, + is -
			return true;
		}
		return false;
	}
	
	/*
	 * Utils
	 */
	
	private static void swapRow(class_310 mc) {
		if (mc.field_1724 == null) {
			reset();
			return;
		}
		if (!mc.field_1724.method_68878()) {
			reset();
			return;
		}
		if (row == 0) {
			reset();
			return;
		}
		
		class_746 player = mc.field_1724;
		class_1661 inv = player.method_31548();
		
		if (singleColumn) {
			swap(mc, inv.method_67532());
		} else {
			for (int i = 0; i < 9; i++) swap(mc, i);
		}
		
		reset();
	}
	
	private static void swap(class_310 mc, int slot) {
		ExtendedInventory.swap(mc, getSlot(row, slot), slot);
	}
	
	/*
	 * Get Utils
	 */
	
	private static int getSlot(int row, int offset) {
		// this math is scary
		return row == 0 ? offset : row < 4 ? ((9 * (4 - row)) + offset) : ((9 * (max + 3 - row)) + offset);
	}
	
	private static class_1799 getMainItem(class_310 mc) {
		if (singleColumn) {
			return ExtendedInventory.getItem(mc, getSlot(row, mc.field_1724.method_31548().method_67532()));
		} else {
			for (int i = 0; i < 9; i++) {
				class_1799 stack = ExtendedInventory.getItem(mc, getSlot(row, i));
				if (!stack.method_7960()) return stack;
			}
			return class_1799.field_8037;
		}
	}
	
}

==> builders_inventory-2.2.6-sources.jar: _3650/builders_inventory/feature/hotbar_swapper/HotbarSwapper.java <==
package _3650.builders_inventory.feature.hotbar_swapper;

import _3650.builders_inventory.BuildersInventory;
import _3650.builders_inventory.ModKeybinds;
import _3650.builders_inventory.config.Config;
import _3650.builders_inventory.feature.extended_inventory.ExtendedInventory;
import _3650.builders_inventory.mixin.feature.hotbar_swapper.GuiGraphicsAccessor;
import _3650.builders_inventory.mixin.feature.hotbar_swapper.GuiInvoker;
import net.minecraft.class_10799;
import net.minecraft.class_1661;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_746;
import net.minecraft.class_9779;

public class HotbarSwapper {
	
	private static final class_2960 SPRITE_HOTBAR = class_2960.method_60656("hud/hotbar");
	private static final class_2960 SPRITE_HOTBAR_SELECTION_WIDE = BuildersInventory.modLoc("hud/hotbar_selection_wide");
	
	public static boolean selecting = false;
	public static boolean singleColumn = false;
	public static int row = 0;
	public static int queueSwap = 0;
	
	public static int max = 4;
	private static int extendedOffset = 2;
	private static int hudShift = (max - 1) * -22 - extendedOffset;
	
	/**
	 * Refreshes the max rows
	 */
	public static void refresh() {
		ExtendedInventory.refresh();
		max = ExtendedInventory.enabled
				&& ExtendedInventory.PAGE_CONTAINER.canModify()
				&& Config.instance().hotbar_swapper_includeExtendedInventory ? 10 : 4;
		extendedOffset = Config.instance().hotbar_swapper_extendedInventoryOffset;
		hudShift = (max - 1) * -22 - extendedOffset;
	}
	
	private static void reset() {
		selecting = false;
		singleColumn = false;
		row = 0;
		queueSwap = 0;
		refresh();
	}
	
	/*
	 * RENDERING
	 */
	
	public static void renderHotbars(class_332 gui, class_310 mc) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			
			if (singleColumn) {
				int offset = mc.field_1724.method_31548().method_67532() * 20;
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 - (i * 22);
					if (i > 3) y -= extendedOffset;
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 182, 22, 0, 0, width - 91 + offset, y, 1, 22);
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 182, 22, 1 + offset, 0, width - 91 + offset + 1, y, 20, 22);
					gui.method_70846(class_10799.field_56883, SPRITE_HOTBAR, 181, 22, 180, 0, width - 91 + offset + 21, y, 1, 22);
				}
			} else {
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 - (i * 22);
					if (i > 3) y -= extendedOffset;
					gui.method_52706(class_10799.field_56883, SPRITE_HOTBAR, width - 91, y, 182, 22);
				}
			}
		}
	}
	
	public static boolean renderHotbarSelector(class_332 gui, class_310 mc, class_2960 selectorSprite) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			if (singleColumn) {
				var sprite = ((GuiGraphicsAccessor) gui).getGuiSprites().method_4608(selectorSprite).method_45851();
				var spriteWidth = sprite.method_45807();
				var spriteHeight = sprite.method_45815();
				
				int offset = mc.field_1724.method_31548().method_67532() * 20;
				int y = gui.method_51443() - 22 - 1 - (row * 22);
				if (row > 3) y -= extendedOffset;
				gui.method_52706(class_10799.field_56883, selectorSprite, width - 91 - 1 + offset, y, spriteWidth, spriteHeight);
			} else {
				int y = gui.method_51443() - 22 - 1 - (row * 22);
				if (row > 3) y -= extendedOffset;
				gui.method_52706(class_10799.field_56883, SPRITE_HOTBAR_SELECTION_WIDE, width - 91 - 1, y, 184, 24);
			}
			
			return true;
		}
		return false;
	}
	
	public static void renderItems(class_332 gui, class_9779 deltaTick, class_310 mc, GuiInvoker hud) {
		if (selecting) {
			int width =  gui.method_51421() / 2;
			class_746 player = mc.field_1724;
			
			if (singleColumn) {
				int x = width - 91 + 3 + (player.method_31548().method_67532() * 20);
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 + 3 - (i * 22);
					if (i > 3) y -= extendedOffset;
					hud.callRenderSlot(gui, x, y, deltaTick, player, ExtendedInventory.getItem(mc, getSlot(i, player.method_31548().method_67532())), x + (y * 22));
				}
			} else {
				for (int i = 1; i < max; i++) {
					int y = gui.method_51443() - 22 + 3 - (i * 22);
					if (i > 3) y -= extendedOffset;
					for (int j = 0; j < 9; j++) {
						int x = width - 91 + 3 + (j * 20);
						hud.callRenderSlot(gui, x, y, deltaTick, player, ExtendedInventory.getItem(mc, getSlot(i, j)), x * y);
					}
				}
			}
		}
	}
	
	public static void renderLabels(class_332 gui, class_310 mc) {
		if (selecting) {
			int width = gui.method_51421() / 2;
			
			int x = singleColumn ? (width - 91 - 7 + (mc.field_1724.method_31548().method_67532() * 20)) : (width - 91 - 7);
			for (int i = 1; i < max; i++) {
				int y = gui.method_51443() - 22 + 7 - (i * 22);
				if (i > 3) y -= extendedOffset;
				gui.method_51433(mc.field_1772, String.valueOf(i), x, y, 0xFFFFFFFF, true);
			}
		}
	}
	
	public static class_1799 toolHighlightOverride(class_1799 stack, class_310 mc, GuiInvoker hud) {
		if (selecting) {
			class_1799 main = getMainItem(mc);
			if (!main.method_7960()) hud.setToolHighlightTimer((int)(40.0 * mc.field_1690.method_48191().method_41753()));
			return main;
		}
		return stack;
	}
	
	public static void shiftHud(class_332 gui) {
		if (selecting) {
			gui.method_51448().pushMatrix();
			gui.method_51448().translate(0, hudShift);
		}
	}
	
	public static void shiftHudReset(class_332 gui) {
		if (selecting) {
			gui.method_51448().popMatrix();
		}
	}
	
	/*
	 * LOGIC
	 */
	
	public static void preKeybinds(class_310 mc) {
		if (selecting && Config.instance().hotbar_swapper_useHotbarHotkeys) {
			// Hotbar keys
			// Consumes all hotbar keys regardless of capacity because I need to cancel them anyways
			for (int i = 0; i < 9; ++i) {
				if (mc.field_1690.field_1852[i].method_1436() && i < max) {
					row = i + 1;
					queueSwap = Config.instance().hotbar_swapper_hotkeyDelay + 1;
				}
			}
		}
	}
	
	public static void clientTick(class_310 mc) {
		if (mc.field_1724 == null) return;
		if (!mc.field_1724.method_68878()) {
			if (selecting) reset();
			return;
		}
		
		if (queueSwap > 0 && --queueSwap == 0) swapRow(mc);
		
		boolean fullKey = ModKeybinds.HOTBAR_SCROLL.method_1434();
		boolean fullKeyPressed = ModKeybinds.HOTBAR_SCROLL.method_1436();
		boolean columnKey = ModKeybinds.COLUMN_SCROLL.method_1434();
		boolean columnKeyPressed = ModKeybinds.COLUMN_SCROLL.method_1436();
		var behavior = Config.instance().hotbar_swapper_behavior;
		
		switch (behavior) {
		case PREFER_FULL:
			if (!Config.instance().hotbar_swapper_stickyModifiers) singleColumn = columnKey;
			if (fullKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = columnKey;
				return;
			}
			if (selecting && !fullKey) swapRow(mc);
			break;
		case PREFER_COLUMN:
			if (!Config.instance().hotbar_swapper_stickyModifiers) singleColumn = !fullKey;
			if (columnKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = !fullKey;
				return;
			}
			if (selecting && !columnKey) swapRow(mc);
			break;
		case SEPARATE:
			if (selecting) {
				if ((!columnKey && singleColumn) || !fullKey && !singleColumn) swapRow(mc);
				return;
			}
			
			if (fullKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = false;
				return;
			}
			if (columnKeyPressed) {
				refresh();
				selecting = true;
				singleColumn = true;
				return;
			}
			break;
		}
	}
	
	public static boolean mouseScroll(double amount) {
		if (selecting) {
			row += amount;
			row %= max;
			if (row < 0) row = max + row; //row is negative, + is -
			return true;
		}
		return false;
	}
	
	/*
	 * Utils
	 */
	
	private static void swapRow(class_310 mc) {
		if (mc.field_1724 == null) {
			reset();
			return;
		}
		if (!mc.field_1724.method_68878()) {
			reset();
			return;
		}
		if (row == 0) {
			reset();
			return;
		}
		
		class_746 player = mc.field_1724;
		class_1661 inv = player.method_31548();
		
		if (singleColumn) {
			swap(mc, inv.method_67532());
		} else {
			for (int i = 0; i < 9; i++) swap(mc, i);
		}
		
		reset();
	}
	
	private static void swap(class_310 mc, int slot) {
		ExtendedInventory.swap(mc, getSlot(row, slot), slot);
	}
	
	/*
	 * Get Utils
	 */
	
	private static int getSlot(int row, int offset) {
		// this math is scary
		return row == 0 ? offset : row < 4 ? ((9 * (4 - row)) + offset) : ((9 * (max + 3 - row)) + offset);
	}
	
	private static class_1799 getMainItem(class_310 mc) {
		if (singleColumn) {
			return ExtendedInventory.getItem(mc, getSlot(row, mc.field_1724.method_31548().method_67532()));
		} else {
			for (int i = 0; i < 9; i++) {
				class_1799 stack = ExtendedInventory.getItem(mc, getSlot(row, i));
				if (!stack.method_7960()) return stack;
			}
			return class_1799.field_8037;
		}
	}
	
}
